package test;

public class TestCryptoMachine2 extends AbstractTest {

    public static void main(String[] args) throws InstantiationException, IllegalAccessException {
        new TestCryptoMachine2();
    }

    protected void runTests() {
        this.testExpected = 19;
        try {
            testSetCryptoStrategyAndGetInformation();
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            testEncode();
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            testDecode();
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            testTest();
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            testGetInformation();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    class MyCode extends crypto.Code {

        public MyCode() {
            super("demo");
        }

        public String encode(String s) {
            return s + "Z";
        }

        public String decode(String s) {
            return s + "T";
        }
    }

    public void testSetCryptoStrategyAndGetInformation() {
        crypto.CryptoMachine p = new crypto.CryptoMachine();

        String expected = "Cesar Algorithm";
        String value = p.getInformation();
        assertTrue(value.contains(expected), "Cesar Algorithm should be set as default coding algorithm");

        p.setCryptoStrategy(new MyCode());
        expected = "demo";
        value = p.getInformation();
        assertTrue(value.contains(expected), "setCryptoStrategy() should change the coding algorithm and therefore the description");
    }

    public void testGetInformation() {
        crypto.CryptoMachine p = new crypto.CryptoMachine();
        p.setCryptoStrategy(new crypto.CesarCodeImpl(13));

        String expected = "Cesar Algorithm\nshift level: 13";
        String value = p.getInformation();
        assertEquals(expected, value);

        p.setCryptoStrategy(new crypto.SecretKeyCodeImpl("ESIAL"));
        expected = "Algorithm using Secret Key\nkey length: 5\nkey: ESIAL";
        value = p.getInformation();
        assertEquals(expected, value);
    }

    public void testEncode() {
        crypto.CryptoMachine m = new crypto.CryptoMachine();

        String msg = "TROP DE SECRET";

        m.setCryptoStrategy(new crypto.SecretKeyCodeImpl("ESIAL"));
        String expected = "XIWPKHWHSPGIMT";
        String value = m.encode(msg);
        assertEquals(expected, value);

        msg = "ESIAL";
        m.setCryptoStrategy(new crypto.CesarCodeImpl(0));
        expected = "ESIAL";
        value = m.encode(msg);
        assertEquals(expected, value);

        m.setCryptoStrategy(new crypto.CesarCodeImpl(1));
        expected = "FTJBM";
        value = m.encode(msg);
        assertEquals(expected, value);

        m.setCryptoStrategy(new crypto.CesarCodeImpl(2));
        expected = "GUKCN";
        value = m.encode(msg);
        assertEquals(expected, value);

        m.setCryptoStrategy(new crypto.CesarCodeImpl(13));
        expected = "REVNY";
        value = m.encode(msg);
        assertEquals(expected, value);
    }

    public void testDecode() {
        crypto.CryptoMachine m = new crypto.CryptoMachine();

        String msg = "XIWPKHWHSPGIMT";

        m.setCryptoStrategy(new crypto.SecretKeyCodeImpl("ESIAL"));
        String expected = "TROP DE SECRET";
        String value = m.decode(msg);
        assertEquals(expected, value);

        msg = "ESIAL";
        m.setCryptoStrategy(new crypto.CesarCodeImpl(0));
        expected = "ESIAL";
        value = m.decode(msg);
        assertEquals(expected, value);

        msg = "FTJBM";
        m.setCryptoStrategy(new crypto.CesarCodeImpl(1));
        expected = "ESIAL";
        value = m.decode(msg);
        assertEquals(expected, value);

        msg = "GUKCN";
        m.setCryptoStrategy(new crypto.CesarCodeImpl(2));
        expected = "ESIAL";
        value = m.decode(msg);
        assertEquals(expected, value);

        msg = "REVNY";
        m.setCryptoStrategy(new crypto.CesarCodeImpl(13));
        expected = "ESIAL";
        value = m.decode(msg);
        assertEquals(expected, value);
    }

    public void testTest() {
        crypto.CryptoMachine m = new crypto.CryptoMachine();

        String msg = "TROP DE SECRET";

        m.setCryptoStrategy(new crypto.SecretKeyCodeImpl("ESIAL"));
        String expected = "TROP DE SECRET -> XIWPKHWHSPGIMT -> TROP DE SECRET";
        String value = m.test(msg);
        assertEquals(expected, value);

        msg = "ESIAL";
        m.setCryptoStrategy(new crypto.CesarCodeImpl(0));
        expected = "ESIAL -> ESIAL -> ESIAL";
        value = m.test(msg);
        assertEquals(expected, value);

        m.setCryptoStrategy(new crypto.CesarCodeImpl(1));
        expected = "ESIAL -> FTJBM -> ESIAL";
        value = m.test(msg);
        assertEquals(expected, value);

        m.setCryptoStrategy(new crypto.CesarCodeImpl(2));
        expected = "ESIAL -> GUKCN -> ESIAL";
        value = m.test(msg);
        assertEquals(expected, value);

        m.setCryptoStrategy(new crypto.CesarCodeImpl(13));
        expected = "ESIAL -> REVNY -> ESIAL";
        value = m.test(msg);
        assertEquals(expected, value);
    }
}
